'use strict';

var path = require('path');
var fs = require('graceful-fs');
var through = require('through2');

/**
 * Recursively resolve a filepath to get the real path for a
 * symlink.
 *
 * @param  {Object} file
 * @param  {NA} `enc` not used
 * @param  {Function} `cb` callback
 * @return {Object}
 */

module.exports = function (app) {
  app.visit('mixin', {

    stream: function() {
      return through.obj(realPath);
    },
  });
};

/**
 * Stream
 */

function realpath(file, enc, cb) {
  var stream = this;

  fs.lstat(file.path, function (err, stat) {
    if (err) {
      stream.emit('error', err);
      process.nextTick(function () {
        cb();
      });
      return;
    }

    file.stat = stat;
    if (!stat.isSymbolicLink()) {
      return cb(null, file);
    }

    fs.realpath(file.path, function (err, fp) {
      if (err) {
        stream.emit('error', err);
        process.nextTick(function () {
          cb();
        });
        return;
      }

      file.base = path.dirname(fp);
      file.path = fp;
      realPath(file, enc, cb);
    });
  });
}